/*
 *	$Source: /home/nlfm/Working/Frink/RCS/config.c,v $
 *	$Date: 2000/08/22 12:25:32 $
 *	$Revision: 2.0.1.2 $
 *
 *------------------------------------------------------------------------
 *   AUTHOR:  Lindsay Marshall <lindsay.marshall@newcastle.ac.uk>
 *------------------------------------------------------------------------
 *    Copyright 2000 The University of Newcastle upon Tyne (see COPYRIGHT)
 *========================================================================
 *
 */

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "frink.h"

extern Token *tokenPop(Token **);
extern void freeToken(Token *);
extern void streamMore(Input *);

List *config = (List *) 0;

static ParamType pcode(char *str)
{
    static struct {
        char		*name;
	ParamType	code;
    } pvals[15] ={
        {"var", PVAR},
	{"code", PCODE},
	{"script", PSCRIPT},
	{"expr", PEXPR},
	{"any", PANY},
	{"single", PSINGLE},
        {"option", POPTION},
	{"flag", PFLAG},
	{"window", PWINDOW},
        {"args", PARGS},
	{"options", POPTIONS},
	{"seqn", PSQN},
	{"builtin", PBLTN},
	{"cmds", PCMDS},
	{"level", PLEVEL},
    };
    int i;

    for (i=0; i < 15; i+= 1)
    {
        if (!strcmp(str, pvals[i].name)) { return pvals[i].code; }
    }
    fprintf(stderr, "\"%s\" is an invalid config item\n", str);
    return PANY;
}

static int handle(Token *line)
{
    Token *hd, *tp, *lp, *lp2, *vp, *vp2;
    ConfigData *cpt;
    ParamData *ppt;
    Input *pfile, *vfile;
    char *txt;

    if (line == (Token *) 0)
    {
	return 1;
    }
    hd = tokenPop(&line);
    switch (hd->type)
    {
    case CONST :
	cpt = (ConfigData *) malloc(sizeof(ConfigData));
	cpt->name = newString(hd->text);
	cpt->params = (List *) 0;
	cpt->oprtn = 0;
	lpush(&config, cpt);
	tp = line;
	if (tp != (Token *) 0)
	{
	    switch (tp->type)
	    {
	    case CONST:
		ppt = (ParamData *) malloc(sizeof(ParamData));
		ppt->values = (List *) 0;
		ppt->optional = 0;
		txt = tp->text;
		if (*txt == '?')
		{
		    ppt->optional = 1;
		    txt++;
		}
		ppt->type = pcode(txt);
		lpush(&cpt->params, ppt);
		break;
	    case LIST:
	        pfile = tokenise(tp, 0);
		lp2 = lp = accumulate(pfile, 1);
		while (lp != (Token *) 0)
		{
		    ppt = (ParamData *) malloc(sizeof(ParamData));
		    ppt->values = (List *) 0;
		    ppt->optional = 0;
		    lappend(&cpt->params, ppt);
	    	    switch (lp->type)
		    {
		    case CONST:
		        txt = lp->text;
		        if (*txt == '?')
		        {
		            ppt->optional = 1;
		            txt++;
		        }
			ppt->type = pcode(txt);
		    	break;

		    case LIST:
	        	vfile = tokenise(lp, 0);
			vp2 = vp = accumulate(vfile, 1);
			ppt->type = pcode(vp->text);
			vp = vp->next;
			while (vp != (Token *) 0)
			{
			    switch (vp->type)
			    {
			    case CONST:
			        if (ppt->type == PBLTN)
				{
				    cpt->oprtn = atoi(vp->text);
				}
			        lpush(&ppt->values, newString(vp->text));
			        break;
			    default:
			        break;
			    }
			    vp = vp->next;
			}
			untokenise(vfile);
			freeToken(vp2);
		        break;

		    case ENDF:
		        break;

		    default:
		        fprintf(stderr, "Error in config\n");
		    	exit(0);
		    }
		    lp = lp->next;
		}
		untokenise(pfile);
		freeToken(lp2);
		break;
	    default:
		fprintf(stderr, "Error in config file\n");
		exit(1);
	    }
	}
	break;
    case PRAGMA :
    case COMMENT :
	break;
    case SCOMMENT :
	break;
    case SEMI :
    case NL:
	break;
    default :
	fprintf(stderr, "Warning: error in config file\n");
	exit(1);
    case ENDF :
	freeToken(hd);
	return 0;
    }
    freeToken(hd);
    freeToken(line);
    return 1;
}

void readconfig(char *str)
{
    FILE *fd;
    Input file;
    
    if ((fd = fopen(str, "r")) == NULL)
    {
	fprintf(stderr, "Warning: cannot open config file\n");
	return;
    }
/*
 * use the tokenising mechanism we already have to parse the config file
 */
    file.text = (char *) malloc(64*1024);
    file.stream = fd;
    file.tcall = file.texpr = 0;
    file.lineNumber = 1;
    file.lineStart = 1;
    streamMore(&file);
    while(handle(collect(&file)))
    {
    }
    free(file.text);  
    fclose(fd);
}

void stringconfig(char *str)
{
    Input file;
/*
 * use the tokenising mechanism we already have to parse the config string
 */
    file.position = file.text = str;
    file.remaining = strlen(str);
    file.pushed = 0;
    file.stream = NULL;
    file.tcall = file.texpr = 0;
    file.lineNumber = 1;
    file.lineStart = 1;
    while(handle(collect(&file)))
    {
    }
}
