/*
 *	$Source: /home/nlfm/Working/Frink/RCS/frink.h,v $
 *	$Date: 2000/08/22 12:25:32 $
 *	$Revision: 2.0.1.2 $
 *
 *------------------------------------------------------------------------
 *   AUTHOR:  Lindsay Marshall <lindsay.marshall@newcastle.ac.uk>
 *------------------------------------------------------------------------
 *    Copyright 1994-2000 The University of Newcastle upon Tyne (see COPYRIGHT)
 *========================================================================
 *
 */
 
#define VMAJOR	"2"
#define VMINOR	"0"
#define VSUB	"0"
#define VPATCH	"0"

extern int internat;
extern int extract;
extern int olcomments;
extern int doTime;
extern int elseif;
extern int doCatch;
extern int doExpr;
extern int lineNumber;
extern int lmargin;
extern int switchIn;
extern int width;
extern int addSpaces;
extern int indent;
extern int contdent;
extern int nocomments;
extern int obfuscate;
extern int tabsOn;
extern int putElse;
extern int putThen;
extern int tabStops;
extern int tclX;
extern int minimise;
extern int xf;
extern int oneliner;
extern int doBind;
extern int haltonWarn;
extern int debrace;
extern int trace;
extern int varbrace;
extern int noblanks;
extern int embrace;
extern int procnls;
extern int ifelse;
extern int nonlelsif;
extern int testonly;
extern int heuristics;
extern int nocommand;
extern int trystrings;

extern char *style;
extern char *pstyle;
extern char *contString;

extern char *locale;
extern FILE *msgfile;

extern int pragma;

typedef struct list_s
{
    struct list_s	*next;
    void		*item;
} List;

extern void lpush(List **, void *);
extern void *lpop(List **);
extern void lappend(List **, void*);
extern void ldel(List **, int);
extern void lapply(List *, void(*)(void *));

extern char *newString(char *);

typedef struct input_s
{
    char *position;
    int	 remaining;
    int  pushed;
    char back[4];
    char *text;
    FILE *stream;
    int tcall;
    int texpr;
    int lineNumber;
    int lineStart;
} Input;

typedef enum token_e {
    ENDF, ENDLINE, ENDC, LIST, STRING, CONC, CALL, VAR, CPART, SPART,
    CONST, ARRAY, NL, SP, COMMENT, HEAD, LBRACK, LBRACE, LPAREN, DOLLAR,
    VNAME, OSTART, START, XCONT, CONT, RBRACK, RBRACE, BLANK, DQSTART,
    DQEND, SEMI, EM, EN, ECONT, SLIST, SCOMMENT, NOSP, SRBRACE,
    RPAREN, SLBRACE, PRAGMA
} TokenType;

typedef enum const_e
{
    CNSTWORD, CNSTSTRING, CNSTLIST
} ConstType;

typedef struct token_s
{
    TokenType type;
    ConstType ckind;
    int isconc;
    char *text;
    int length;
    struct token_s *sequence;
    struct token_s *next;
    char little[32];
    int lineNo;
} Token;

enum Heuristics
{
    HVAR	= 0x0001,	/* check for non-var 1st param		*/
    HNUMBER	= 0x0002,	/* ditto				*/
    HVALUE	= 0x0004,	/* ditto				*/
    HRETURN	= 0x0010,	/* check for value/novalue returns	*/
    HNSNAME	= 0x0020,	/* check for : instead of :: in names	*/
    HEXPR	= 0x0040,	/* check various expr issues		*/
    HFOREACH	= 0x0080,	/* check various foreach issues		*/
    HNOPARAM	= 0x0100,	/* check missing parameters		*/
    HALL	= 0xffff
};

enum Pragmas
{
    NOTREACHED	= 0x0001,
    NOCHECK	= 0x0002,
    NOFORMAT	= 0x0004
};

typedef struct procret_s
{
    int returns;
    int result;
    Token *pname;
} Procret;

typedef enum ptype
{
    PVAR, PCODE, PSCRIPT, PEXPR, PANY, PSINGLE, POPTION, PFLAG,
    PWINDOW, PARGS, POPTIONS, PSQN, PBLTN, PCMDS, PLEVEL
} ParamType;

typedef struct param_s
{
    ParamType	type;
    int		optional;
    List	*values;
} ParamData;

typedef struct config_s
{
    char 	*name;
    List	*params;
    int		oprtn;
} ConfigData;

extern List *config;

extern List *skiplist;
extern List *retinfo;

extern Input *tokenise(Token *, int);
extern void untokenise();

extern Token *collect(Input *);
extern Token *accumulate(Input *, int);
extern Token *getToken(Input *);
extern void tokenPush(Token **tp, Token *v);
extern void freeToken(Token *);
extern void lprocess(Token*, int);
extern void makeCall(Token*, Token*);
extern void body(Token *, int);

extern void output(Token *, int);
extern void blank();
extern void warn(Token *, char *);
extern void fail(Token *, char *);
